__doc__ = '''Calendar tag'''
__version__='1.0.7'

from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String
from DateTime.DateTime import DateTime
from string import capitalize, join, replace, split
from types import StringType

##from time import time

start = """
<table border=0 bgcolor="%(bgcolor)s" cellspacing=0 cellpadding=0>
 <tr><td><table border=0 cellspacing=0 cellpadding=0>
 <tr>
  <td align="center" valign="middle" bgcolor="%(leftbgcolor)s">
   <font color="%(leftfgcolor)s">%(left)s</font></td>
  %(middle)s
  %(right)s
 </tr>
 </table></td></tr>
"""

end = """
</table>
"""

body_start = """
 <tr>
  <td bgcolor="%(bordercolor)s" align="center" valign="middle" colspan="%(colspan)s">
   <table border=0 width="100%%" cellspacing="%(spacing)s" cellpadding="%(padding)s">
"""

body_end = """
   </table>
  </td>
 </tr>
"""

right = """
  <td align="right" valign="top" bgcolor="%(rightbgcolor)s">
   <table border=0 cellspacing=0 cellpadding=0><tr>
    <td>&nbsp;</td>
    <td><a href="%(prev_url_)s"><img border=0 alt="%(prev_alt_)s" src="%(ipath_)slarrow"></a></td>
    <td>%(rightmiddle)s</td>
    <td><a href="%(next_url_)s"><img border=0 alt="%(next_alt_)s" src="%(ipath_)srarrow"></a></td>
   </tr></table>
  </td>
"""

## Translate machinery
daynames = {'ca': ('Du', 'Dll', 'Dm', 'Dx', 'Dj', 'Dv', 'Ds'),
            'de': ('So', 'Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa'),
            'du': ('Zo', 'M', 'Di', 'W', 'Do', 'V', 'Za'),
            'en': 'SMTWTFS',
            'es': ('D', 'L', 'M', 'Mi', 'J', 'V', 'S'),
            'fi': ('Su', 'Ma', 'Ti', 'Ke', 'To', 'Pe', 'La'),
            'fr': ('D', 'L', 'M', 'M', 'J', 'V', 'S'),
            'it': ('Do', 'Lu', 'Ma', 'Me', 'Gio', 'Ve', 'Sa'),
            'no': 'SMTOTFL',
            'pt': ('Dom', 'Seg', 'Ter', 'Qua', 'Qui', 'Sex', 'Sb'),
            'zh_TW': ('g', 'g@', 'gG', 'gT', 'g|', 'g', 'g'),
            'ru': ('', '', '', '', '', '', '')}

translations = {
    'Day'  : {'ca': 'Dia',
              'de': 'Tag',
              'du': 'Dag',
              'es': 'D&iacute;a',
              'fi': 'Piv',
              'fr': 'Jour',
              'it': 'Giorno',
              'no': 'Dag',
              'pt': 'Dia',
              'zh_TW': '@',
              'ru': ''},
    'Week' : {'ca': 'Setmana',
              'de': 'Woche',
              'du': 'Week',
              'es': 'Semana',
              'fi': 'Viikko',
              'fr': 'Semaine',
              'it': 'Settimana',
              'no': 'Uke',
              'pt': 'Semana',
              'zh_TW': '@g',
              'ru': ''},
    'Month': {'ca': 'Mes',
              'de': 'Monat',
              'du': 'Maand',
              'es': 'Mes',
              'fi': 'Kuukausi',
              'fr': 'Mois',
              'it': 'Mese',
              'no': 'M&aring;ned',
              'pt': 'Ms',
              'zh_TW': '@',
              'ru': ''},
    'Year' : {'ca': 'Any',
              'de': 'Jahr',
              'du': 'Jaar',
              'es': 'Ao',
              'fi': 'Vuosi',
              'fr': 'Ann&eacute;e',
              'it': 'Anno',
              'no': '&Aring;r',
              'pt': 'Ano',
              'zh_TW': '@~',
              'ru': ''},
    'Previous Week' : {'ca': 'Setmana previa',
                       'de': 'Vorige Woche',
                       'du': 'Vorige Week',
                       'es': 'Semana previa',
                       'fi': 'Edellinen viikko',
                       'fr': 'Semaine Pr&eacute;c&eacute;dente',
                       'it': 'Settimana precedente',
                       'no': 'Forrige Uke',
                       'pt': 'Semana Anterior',
                       'zh_TW': 'W@g',
                       'ru': ' '},
    'Next Week'     : {'ca': 'Setmana seg&uuml;ent',
                       'de': 'N&auml;chste Woche',
                       'du': 'Volgende Week',
                       'es': 'Semana siguiente',
                       'fi': 'Seuraava viikko',
                       'fr': 'Semaine Suivante',
                       'it': 'Settimana successiva',
                       'no': 'Neste Uke',
                       'pt': 'Prxima Semana',
                       'pt': 'U@g',
                       'ru': ' '},
    'Previous Month': {'ca': 'Mes previ',
                       'de': 'Voriger Monat',
                       'du': 'Vorige Maand',
                       'es': 'Mes previo',
                       'fi': 'Edellinen kuukausi',
                       'fr': 'Mois Pr&eacute;c&eacute;dent',
                       'it': 'Mese precedente',
                       'no': 'Forrige M&aring;ned',
                       'pt': 'Ms Anterior',
                       'zh_TW': 'WӤ',
                       'ru': ' '},
    'Next Month'    : {'ca': 'Mes seg&uuml;ent',
                       'de': 'N&auml;chster Monat',
                       'du': 'Volgende Maand',
                       'es': 'Mes siguiente',
                       'fi': 'Seuraava kuukausi',
                       'fr': 'Mois Suivant',
                       'it': 'Mese successivo',
                       'no': 'Neste M&aring;ned',
                       'pt': 'Prximo Ms',
                       'zh_TW': 'UӤ',
                       'ru': ' '},
    'Previous Year' : {'ca': 'Any previ',
                       'de': 'Voriges Jahr',
                       'du': 'Vorig Jaar',
                       'es': 'A&ntilde;o previo',
                       'fi': 'Edellinen vuosi',
                       'fr': 'Ann&eacute;e Pr&eacute;c&eacute;dente',
                       'it': 'Anno precedente',
                       'no': 'Forrige &Aring;r',
                       'pt': 'Ano Anterior',
                       'zh_TW': 'W@~',
                       'ru': ' '},
    'Next Year'     : {'ca': 'Any seg&uuml;ent',
                       'de': 'N&auml;chstes Jahr',
                       'du': 'Volgend Jaar',
                       'es': 'A&ntilde;o siguiente',
                       'fi': 'Seuraava vuosi',
                       'fr': 'Ann&eacute;e Suivante',
                       'it': 'Anno successivo',
                       'no': 'Neste &Aring;r',
                       'pt': 'Prximo Ano',
                       'zh_TW': 'U@~',
                       'ru': ' '},
    'January'  : {'ca': 'Gener',
                  'de': 'Januar',
                  'du': 'Januari',
                  'es': 'Enero',
                  'fi': 'Tammikuu',
                  'fr': 'Janvier',
                  'it': 'Gennaio',
                  'no': 'Januar',
                  'pt': 'Janeiro',
                  'zh_TW': '@',
                  'ru': ''},
    'February' : {'ca': 'Febrer',
                  'de': 'Februar',
                  'du': 'Februari',
                  'es': 'Febrero',
                  'fi': 'Helmikuu',
                  'fr': 'F&eacute;vrier',
                  'it': 'Febbraio',
                  'no': 'Februar',
                  'pt': 'Fevereiro',
                  'zh_TW': 'G',
                  'ru': ''},
    'March'    : {'ca': 'Mar&ccedil;',
                  'de': 'M&auml;rz',
                  'du': 'Maart',
                  'es': 'Marzo',
                  'fi': 'Maaliskuu',
                  'fr': 'Mars',
                  'it': 'Marzo',
                  'no': 'Mars',
                  'pt': 'Maro',
                  'zh_TW': 'T',
                  'ru': ''},
    'April'    : {'ca': 'Abril',
                  'de': 'April',
                  'du': 'April',
                  'es': 'Abril',
                  'fi': 'Huhtikuu',
                  'fr': 'Avril',
                  'it': 'Aprile',
                  'no': 'April',
                  'pt': 'Abril',
                  'zh_TW': '|',
                  'ru': ''},
    'May'      : {'ca': 'Maig',
                  'de': 'Mai',
                  'du': 'Mei',
                  'es': 'Mayo',
                  'fi': 'Toukokuu',
                  'fr': 'Mai',
                  'it': 'Maggio',
                  'no': 'Mai',
                  'pt': 'Maio',
                  'zh_TW': '',
                  'ru': ''},
    'June'     : {'ca': 'Juny',
                  'de': 'Juni',
                  'du': 'Juni',
                  'es': 'Junio',
                  'fi': 'Keskuu',
                  'fr': 'Juin',
                  'it': 'Giugno',
                  'no': 'Juni',
                  'pt': 'Junho',
                  'zh_TW': '',
                  'ru': ''},
    'July'     : {'ca': 'Juliol',
                  'de': 'Juli',
                  'du': 'Juli',
                  'es': 'Julio',
                  'fi': 'Heinkuu',
                  'fr': 'Juillet',
                  'it': 'Luglio',
                  'no': 'Juli',
                  'pt': 'Julho',
                  'zh_TW': 'C',
                  'ru': ''},
    'August'   : {'ca': 'Agost',
                  'de': 'August',
                  'du': 'Augustus',
                  'es': 'Agosto',
                  'fi': 'Elokuu',
                  'fr': 'Ao&ucirc;t',
                  'it': 'Agosto',
                  'no': 'August',
                  'pt': 'Agosto',
                  'zh_TW': 'K',
                  'ru': ''},
    'September': {'ca': 'Setembre',
                  'de': 'September',
                  'du': 'September',
                  'es': 'Septiembre',
                  'fi': 'Syyskuu',
                  'fr': 'Septembre',
                  'it': 'Settembre',
                  'no': 'September',
                  'pt': 'Setembro',
                  'zh_TW': 'E',
                  'ru': ''},
    'October'  : {'ca': 'Octubre',
                  'de': 'Oktober',
                  'du': 'October',
                  'es': 'Octubre',
                  'fi': 'Lokakuu',
                  'fr': 'Octobre',
                  'it': 'Ottobre',
                  'no': 'Oktober',
                  'pt': 'Outubro',
                  'zh_TW': 'Q',
                  'ru': ''},
    'November' : {'ca': 'Novembre',
                  'de': 'November',
                  'du': 'November',
                  'es': 'Noviembre',
                  'fi': 'Marraskuu',
                  'fr': 'Novembre',
                  'it': 'Novembre',
                  'no': 'November',
                  'pt': 'Novembro',
                  'zh_TW': 'Q@',
                  'ru': ''},
    'December' : {'ca': 'Desembre',
                  'de': 'Dezember',
                  'du': 'December',
                  'es': 'Diciembre',
                  'fi': 'Joulukuu',
                  'fr': 'D&eacute;cembre',
                  'it': 'Dicembre',
                  'no': 'Desember',
                  'pt': 'Dezembro',
                  'zh_TW': 'QG',
                  'ru': ''},
    'Jan': {'ca': 'Gen',
            'de': 'Jan',
            'du': 'Jan',
            'es': 'Ene',
            'fi': 'Tam',
            'fr': 'Jan',
            'it': 'Gen',
            'no': 'Jan',
            'pt': 'Jan',
            'zh_TW': '@',
            'ru': ''},
    'Feb': {'ca': 'Feb',
            'de': 'Feb',
            'du': 'Feb',
            'es': 'Feb',
            'fi': 'Hel',
            'fr': 'F&eacute;v',
            'it': 'Feb',
            'no': 'Feb',
            'pt': 'Fev',
            'zh_TW': 'G',
            'ru': ''},
    'Mar': {'ca': 'Mar',
            'de': 'M&auml;r',
            'du': 'Maa',
            'es': 'Mar',
            'fi': 'Maa',
            'fr': 'Mar',
            'it': 'Mar',
            'no': 'Mar',
            'pt': 'Mar',
            'zh_TW': 'T',
            'ru': ''},
    'Apr': {'ca': 'Abr',
            'de': 'Apr',
            'du': 'Apr',
            'es': 'Abr',
            'fi': 'Huh',
            'fr': 'Avr',
            'it': 'Apr',
            'no': 'Apr',
            'pt': 'Abr',
            'zh_TW': '|',
            'ru': ''},
    'May': {'ca': 'Mai',
            'de': 'Mai',
            'du': 'Mei',
            'es': 'May',
            'fi': 'Tou',
            'fr': 'Mai',
            'it': 'Mag',
            'no': 'Mai',
            'pt': 'Mai',
            'zh_TW': '',
            'ru': ''},
    'Jun': {'ca': 'Jun',
            'de': 'Jun',
            'du': 'Jun',
            'es': 'Jun',
            'fi': 'Kes',
            'fr': 'Juin',
            'it': 'Giu',
            'no': 'Jun',
            'pt': 'Jun',
            'zh_TW': '',
            'ru': ''},
    'Jul': {'ca': 'Jul',
            'de': 'Jul',
            'du': 'Jul',
            'es': 'Jul',
            'fi': 'Hei',
            'fr': 'Juil',
            'it': 'Lug',
            'no': 'Jul',
            'pt': 'Jul',
            'zh_TW': 'C',
            'ru': ''},
    'Aug': {'ca': 'Ago',
            'de': 'Aug',
            'du': 'Aug',
            'es': 'Ago',
            'fi': 'Elo',
            'fr': 'Ao&ucirc;',
            'it': 'Ago',
            'no': 'Aug',
            'pt': 'Ago',
            'zh_TW': 'K',
            'ru': ''},
    'Sep': {'ca': 'Set',
            'de': 'Sep',
            'du': 'Sep',
            'es': 'Sep',
            'fi': 'Syy',
            'fr': 'Sep',
            'it': 'Set',
            'no': 'Sep',
            'pt': 'Set',
            'zh_TW': 'E',
            'ru': ''},
    'Oct': {'ca': 'Oct',
            'de': 'Okt',
            'du': 'Oct',
            'es': 'Oct',
            'fi': 'Lok',
            'fr': 'Oct',
            'it': 'Ott',
            'no': 'Okt',
            'pt': 'Out',
            'zh_TW': 'Q',
            'ru': ''},
    'Nov': {'ca': 'Nov',
            'de': 'Nov',
            'du': 'Nov',
            'es': 'Nov',
            'fi': 'Mar',
            'fr': 'Nov',
            'it': 'Nov',
            'no': 'Nov',
            'pt': 'Nov',
            'zh_TW': 'Q@',
            'ru': ''},
    'Dec': {'ca': 'Des',
            'de': 'Dez',
            'du': 'Dec',
            'es': 'Dic',
            'fi': 'Jou',
            'fr': 'D&eacute;c',
            'it': 'Dec',
            'no': 'Des',
            'pt': 'Dez',
            'zh_TW': 'QG',
            'ru': ''},
##    'Sunday'   : {'ca': 'Diumenge',
##                  'de': 'Sonntag',
##                  'du': 'Zondag',
##                  'es': 'Domingo',
##                  'fi': 'Sunnuntai',
##                  'fr': 'Dimanche',
##                  'it': '',
##                  'no': 'S&oslash;ndag',
##                  'pt': 'Domingo',
##                  'ru': ''},
##    'Monday'   : {'ca': 'Dilluns',
##                  'de': 'Montag',
##                  'du': 'Maandag',
##                  'es': 'Lunes',
##                  'fi': 'Maanantai',
##                  'fr': 'Lundi',
##                  'it': '',
##                  'no': 'Mandag',
##                  'pt': 'Segunda',
##                  'ru': ''},
##    'Tuesday'  : {'ca': 'Dimarts',
##                  'de': 'Dienstag',
##                  'du': 'Dinsdag',
##                  'es': 'Martes',
##                  'fi': 'Tiistai',
##                  'fr': 'Mardi',
##                  'it': '',
##                  'no': 'Tirsdag',
##                  'pt': 'Tera',
##                  'ru': ''},
##    'Wednesday': {'ca': 'Dimecres',
##                  'de': 'Mittwoch',
##                  'du': 'Woensdag',
##                  'es': 'Mi&eacute;rcoles',
##                  'fi': 'Keskiviikko',
##                  'fr': 'Mercredi',
##                  'it': '',
##                  'no': 'Onsdag',
##                  'pt': 'Quarta',
##                  'ru': ''},
##    'Thursday' : {'ca': 'Dijous',
##                  'de': 'Donnerstag',
##                  'du': 'Donderdag',
##                  'es': 'Jueves',
##                  'fi': 'Torstai',
##                  'fr': 'Jeudi',
##                  'it': '',
##                  'no': 'Torsdag',
##                  'pt': 'Quinta',
##                  'ru': ''},
##    'Friday'   : {'ca': 'Divendres',
##                  'de': 'Freitag',
##                  'du': 'Vrijdag',
##                  'es': 'Viernes',
##                  'fi': 'Perjantai',
##                  'fr': 'Vendredi',
##                  'it': '',
##                  'no': 'Fredag',
##                  'pt': 'Sexta',
##                  'ru': ''},
##    'Saturday' : {'ca': 'Dissabte',
##                  'de': 'Samstag',
##                  'du': 'Zaterdag',
##                  'es': 'S&aacute;bado',
##                  'fi': 'Lauantai',
##                  'fr': 'Samedi',
##                  'it': '',
##                  'no': 'L&oslash;rdag',
##                  'pt': 'Sbado',
##                  'ru': ''}
                }

def translate(word, lang):
    try:
        return translations[word][lang]
    except KeyError:
        return word


def datestr(date):
    return '%d-%s-%s' % (date.year(), date.mm(), date.dd())


def name_or_expr(mapping, name_attr, expr_attr, default):
    name = mapping.get(name_attr, None)
    expr = mapping.get(expr_attr, None)

    if name is None:
        if expr is None:
            return default
        return VSEval.Eval(expr, expr_globals)
    if expr is None:
        return name
    raise ParseError, ('%s and %s given' % (name_attr, expr_attr), 'calendar')

def get_value(x, md):
    if type(x) is StringType:
        return x
    else:
        return x.eval(md)


class CalendarTag:
    """
    Calendar tag.
    
    See README.txt for usage, author, copyright, and license.
    """

    name = 'calendar'
    blockContinuations = ()
    expr=None

    def __init__(self, blocks):
        tname, args, section = blocks[0]
        self.section = section.blocks
        self.args = parse_params(args, name='', expr='',
                                 bgcolor='', mode='', date='',
                                 controls='', weekdays='',
                                 lang='', lang_expr='',
                                 modes='')

        self.__name__ = name_or_expr(self.args, 'name', 'expr', 'calendar')

        modes = self.args.get('modes', None)
        if modes is None:
            self.modes = ('day', 'week', 'month', 'year')
        else:
            self.modes = split(modes, ',')
            for mode in self.modes:
                if mode not in ('day', 'week', 'month', 'year'):
                    raise "Unallowed mode: %s" % mode

        self.bgcolor = self.args.get('bgcolor', '#ffffff')
        self.mode = self.args.get('mode', 'month')
        if self.mode not in self.modes:
            if 'month' in self.modes:
                self.mode = 'month'
            else:
                self.mode = self.modes[0]
        self.date = self.args.get('date', None)

        self.controls = self.args.get('controls','yes')

        weekdays = self.args.get('weekdays', '07')
        self.week_sday = int(weekdays[0])
        self.week_ndays = int(weekdays[1])

        self.__lang__ = name_or_expr(self.args, 'lang', 'lang_expr', 'en')

    def render(self, md):
##        time_start = time()

        name = get_value(self.__name__, md)
        lang = get_value(self.__lang__, md)

        if md.has_key('date-' + name) and self.controls != 'no':
            date = DateTime(md['date-' + name])
        else:
            if self.date:
                d = Eval(self.date, expr_globals)
                d = d.eval(md)
                if type(d) in (type(0), type('')):
                    date = DateTime(d)
                else:
                    date = DateTime(str(d))
            else:
                date = DateTime()

        date = date.earliestTime() + 0.5

        if md.has_key('mode-' + name) and self.controls != 'no':
            mode = md['mode-' + name]
        else:
            mode = self.mode

        self.mode = mode
        if mode not in self.modes:
            raise "Unallowed mode: %s" % mode

        self.vals_ = v = {}

        ## Translations
        for word in translations.keys():
            v[word] = translate(word, lang)


        v['name_'] = name
        v['date_'] = datestr(date)
        v['date'] = date
        v['mode'] = mode

        v['spacing'] = '2'
        v['padding'] = '4'
        v['bordercolor'] = '#000000'

        v['bgcolor'] = v['leftfgcolor'] = v['middlebgcolor'] = \
            v['rightbgcolor'] = v['daybgcolor'] = v['mhbgcolor'] = self.bgcolor
        v['leftbgcolor'] = '#000000'

        v['ipath_'] = md['BASE1'] + '/misc_/Calendar/'

        l = ['mode-' + name, 'date-' + name]
        params = filter(lambda x: x, split(md['QUERY_STRING'], '&'))
        params = map(lambda x: split(x, '='), params)
        params = filter(lambda x, l=l: x[0] not in l, params)
        params = map(lambda x: join(x, '='), params)

        v['url_'] = md['URL'] + '?' + join(params, '&amp;')

        v['day_img_'] = 'day'
        v['week_img_'] = 'week'
        v['month_img_'] = 'month'
        v['year_img_'] = 'year'

        v['rightmiddle'] = '&nbsp;'

        v['prev_alt_'] = v['Previous Week']
        v['prev_url_'] = self.linkDate_(date - 7, mode)

        v['next_alt_'] = v['Next Week']
        v['next_url_'] = self.linkDate_(date + 7, mode)

        try:
            v['daynames'] = daynames[lang]
        except KeyError:
            v['daynames'] = daynames['en']

        ##
        if self.controls == 'no':
            v['colspan'] = 1
        else:
            v['colspan'] = 3


        ## Calculate the body
        if mode == 'year':
            s = '<tr>'
            d = date - (date.dayOfYear() - 1)
            for i in range(1,13):
                body = self.render_dwm('month', md, d)
                s = s + '<td valign="top"><table CELLPADDING="0" CELLSPACING="0" BORDER="0">'
                s = s + (body_start % v) +\
                    '<tr><th bgcolor="%s" colspan="%d">'\
                    % (v['mhbgcolor'], self.week_ndays)
                if self.controls != 'no':
                    s = s + '<a href="%s">' % self.linkDate_(d, 'month')
                s = s + v[d.Month()]
                if self.controls != 'no':
                    s = s + '</a>'
                s = s + '</th></tr>' + body + body_end + '</table></td>'
                if i%3 == 0:
                    s = s + '</tr><tr><td>&nbsp;</td></tr><tr>'
                else:
                    s = s + '<td> &nbsp; </td>'
                d = d + 40
                d = d - (d.day() - 1)
            s = s + '</tr>'
        else:
            body = self.render_dwm(mode, md, date)
            s = (body_start % v) + body + body_end

        ## Calculate left
        if mode == 'year':
            left = str(date.year())
            v['year_img_'] = 'syear'
            v['prev_alt_'] = v['Previous Year']
            v['next_alt_'] = v['Next Year']

            d = date - date.dayOfYear()
            v['prev_url_'] = self.linkDate_(d, mode)
            if date.isLeapYear():
                d = date + 366
            else:
                d = date + 365
            v['next_url_'] = self.linkDate_(d, mode)
        else:
            if mode == 'day':
                monthname = v[date.aMonth()]
                left = '%s %d, %d' % (monthname, date.day(), date.year())
                v['day_img_'] = 'sday'
            elif mode == 'week':
                sdow = date - date.dow()
                edow = sdow + 6
                smonthname = v[sdow.aMonth()]
                emonthname = v[edow.aMonth()]

                left = '%s %d' % (emonthname, edow.year())
                if sdow.aMonth() != edow.aMonth():
                    left = ' - ' + left
                    if sdow.year() != edow.year():
                        left = (' %d' % sdow.year()) + left
                    left = smonthname + left

                v['week_img_'] = 'sweek'
            else:
                left = '%s %d' % (v[date.Month()], date.year())
                v['month_img_'] = 'smonth'
                v['prev_alt_'] = v['Previous Month']
                v['next_alt_'] = v['Next Month']

                d = date - date.day()
                v['prev_url_'] = self.linkDate_(d, mode)

                d = date - (date.day() - 40)
                v['next_url_'] = self.linkDate_(d, mode)

        v['left'] = '&nbsp;&nbsp;<strong>' + left + '</strong>&nbsp;&nbsp;'

        ## Calculate middle and right
        if self.controls == 'no':
            v['middle'] = ''
            v['right'] = ''
        else:
            if mode == 'day':
                rmid = '<TD VALIGN="top"><TABLE BGCOLOR="#000000" CELLSPACING="0" CELLPADDING="0" BORDER="0"><TR><TD><TABLE BGCOLOR="#000000" CELLSPACING="1" CELLPADDING="1" BORDER="0"><TR>'
                if date.dow() >= self.week_sday:
                    d = date - date.dow()
                else:
                    d = date - 7
                d = d + self.week_sday
                for i in range(self.week_ndays):
                    weekday = d.dow()
                    if date.dow() == weekday:
                        color = '#ffffff'
                        bgcolor = '#000000'
                    else:
                        color = '#000000'
                        bgcolor = '#ffffff'

                    v['dow_date_'] = datestr(d)
                    dow_url = "%(url_)s&amp;date-%(name_)s=%(dow_date_)s&amp;mode-%(name_)s=%(mode)s" % v

                    rmid = rmid + '<td BGCOLOR="%s"><a STYLE="text-decoration: none; color: %s" HREF="%s"><big><big><strong>%s</strong></big></big></a></td>' % (bgcolor, color, dow_url, daynames[lang][weekday])
                    d = d + 1

                v['rightmiddle'] = rmid +'</tr></table></td></tr></table></td>'
            elif mode == 'week':
                v['rightmiddle'] = '&nbsp;%s %d&nbsp;' %\
                                   (v['Week'] , date.dayOfYear() / 7 + 1)

            if not v.has_key('middle'):
                middle = '<td align="center" valign="top" bgcolor="%(middlebgcolor)s"><table border=0 cellspacing=0 cellpadding=0><tr><td>&nbsp;</td>'
                for mode in self.modes:
                    middle = middle + '<td><a href="%s&amp;date-%s=%s&amp;mode-%s=%s"><img border=0 src="%s%s" alt="%s"></a></td>' % (replace(v['url_'], '%', '%%'), v['name_'], v['date_'], v['name_'], mode, v['ipath_'], v[mode+'_img_'], v[capitalize(mode)])
                v['middle'] = middle + "</tr></table></td>"
            v['middle'] = v['middle'] % v

            if not v.has_key('right'):
                v['right'] = right
            v['right'] = v['right'] % v


##        time_end = time()
##        time_diff = time_end - time_start

        return (start % v) + s + end ## + ("<!-- %f -->" % time_diff)

    __call__ = render


    def render_dwm(self, mode, md, date):
        v = self.vals_

        if mode == 'day':
            s = '<tr>%s</tr>' % self.render_day_(md, date, 1)
        else:
            s = '<tr>'
            if mode == 'week':
                while date.dow() != self.week_sday:
                    date = date - 1
                for i in range(self.week_ndays):
                    s = s + self.render_day_(md, date + i, i == 0)
            elif mode == 'month':
                bd = date - (date.day() - 1)
                c = 0
                while (bd - c).dow() != self.week_sday:
                    c = c + 1
                if c > 0:
                    s = s + '<td bgcolor="%s" colspan="%s">&nbsp;</td>' % (
                        self.bgcolor, c)

                d = bd
                month = bd.month()
                while d.month() == month:
                    if c == self.week_ndays:
                        s = s + '</tr><tr>'
                        c = 0
                    s = s + self.render_day_(md, d, d == bd)
                    d = d + 1
                    c = c + 1
                if c < self.week_ndays:
                    s = s + '<td bgcolor="%s" colspan="%s">&nbsp;</td>' % (
                        self.bgcolor, self.week_ndays - c)
            s = s + '</tr>'

            if v['daynames']:
                x = ''
                c = v['daybgcolor']

                for d in range(self.week_sday, self.week_ndays+self.week_sday):
                    x = x + '<th bgcolor="%s" width="%d%%">%s</th>' \
                        % (c, 100/self.week_ndays, v['daynames'][d%7])

                s = '<tr>%s</tr>' % x + s

        return s


    def render_day_(self, md, date, firstday):
        date = date.earliestTime()

        ns = namespace(md,
            date = date,
            mode = self.mode,
            firstday = firstday,
            setCalendar = self.setCalendar_,
            getCalendar = self.getCalendar_,
            linkDate = self.linkDate_
        )[0]

        md._push(InstanceDict(ns, md))

        v = self.vals_
        v['cellbgcolor'] = self.bgcolor
        v['align'] = None
        v['valign'] = None

        v['cellattrs'] = None

        try:
            s = render_blocks(self.section, md)
        finally:
            md._pop(1)
            pass

        x = v['cellattrs']
        if x is None:
            pre = '<td bgcolor="%s"' % v['cellbgcolor']
            x = v['align']
            if x is not None:
                pre = pre + ' align="%s"' % x
            x = v['valign']
            if x is not None:
                pre = pre + ' valign="%s"' % x
        else:
            pre = '<td ' + join(map(lambda x: '%s="%s"' % (x[0],x[1]), x))
        return pre + '>' + s + '</td>\n'


    def linkDate_(self, date, mode = 'day'):
        if mode not in self.modes:
            mode = 'day'

        v = self.vals_
        s = '%(url_)s&amp;date-%(name_)s=' % v
        return s + datestr(date) + ('&amp;mode-%(name_)s=' % v) + mode

        
    def setCalendar_(self, name, val):
        self.vals_[name] = val


    def getCalendar_(self, name, defval = None):
        if self.vals_.has_key(name):
            return self.vals_[name]
        else:
            return defval
	
	

# Install the calendar tag globally

String.commands['calendar'] = CalendarTag

